==> sapdb-source-7.3.00.48.tgz: V73_00_48/SAPDB_ORG/sys/src/co/gco90.h <==
/* @lastChanged: "1998-05-27  11:02"
 * @filename:    gco90.h
 * @purpose:     "DBProcedure: Monitor"
 * @release:     7.1.0.0
 * @see:         ""
 *
 * @copyright:   "Copyright by SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 */


#ifndef GCO90_H
#define GCO90_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define LIVECACHEDBP_VERSION    "LiveCacheDbproc.h 7.1.0.0   DATE: 1998-04-27"

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

struct co90_MonitorInfo {
  tgg00_COMMonitorInfo m_monInfo;
  void Reset() {
    m_monInfo.cmiCallCnt_gg00 = 0;
    for (int ix = cmiRunTime; ix <= cmiCacheSize; ix++) {
      m_monInfo.cmiCounters_gg00[ix] = 0.0;
    }
    m_monInfo.cmiCounters_gg00[cmiRunTimeMin        ] = csp_maxint4;
    m_monInfo.cmiCounters_gg00[cmiMinWaitNewConsView] = csp_maxint4;
  }
  co90_MonitorInfo () {
    Reset();    
  }
};

class co90_Monitor {
public :
  class Iterator {
  public :
    Iterator(co90_Monitor*);
    inline int getDispId() const;
    inline void operator++();
    inline operator bool() const;
    inline co90_MonitorInfo& operator()() const;
  private :
    co90_Monitor* m_monitor;
    int           m_idx;
  };
 
  co90_Monitor(CLSID& IID, long numEntries) : m_size(numEntries), m_iid(IID),  m_currInfo(NULL) {
    /*PTS 1103516 FF*/
    m_monInfoArray = new co90_MonitorInfo[numEntries];
    if (NULL == m_monInfoArray ) 
      m_size = 0;
  }

  /*PTS 1103516 FF*/
  ~co90_Monitor() {
    delete [] m_monInfoArray;
  }

  void Reset () {
    for (int ix = 0; ix < m_size; ix++) {
      (m_monInfoArray+ix)->Reset();
    }
  }

  inline Iterator begin();

  inline void callEpilog(int dispId, int runTime);

  tsp00_Int4 size() {
    return m_size;
  }

  inline tgg00_COMMonitorInfo* getCurrent();

  inline const CLSID& getIID() const;

  inline void setCurrent(int dispId, unsigned char* lpMethodName);

  co90_MonitorInfo& operator[](int idx) {
    return m_monInfoArray[idx];
  }

private :
  co90_MonitorInfo*     m_monInfoArray;   /*PTS 1103516 FF*/
  tsp00_Int4            m_size;
  CLSID                 m_iid;
  tgg00_COMMonitorInfo* m_currInfo;
 
  friend class co90_Monitor::Iterator;
};

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/

inline co90_Monitor::Iterator co90_Monitor::begin() {
  Iterator iter(this);
  return iter;
}

inline void co90_Monitor::callEpilog(int dispId, int runTime)
{
  if ( dispId < size() ) {
    if (runTime > m_currInfo->cmiCounters_gg00[cmiRunTimeMax]) {
      m_currInfo->cmiCounters_gg00[cmiRunTimeMax] = runTime;
    }
    if (runTime < m_currInfo->cmiCounters_gg00[cmiRunTimeMin]) {
      m_currInfo->cmiCounters_gg00[cmiRunTimeMin] = runTime;
    }
    m_currInfo->cmiCounters_gg00[cmiRunTime] += runTime;
    m_currInfo = NULL;
  }
}

inline tgg00_COMMonitorInfo* co90_Monitor::getCurrent() {
  return m_currInfo;
}

inline const CLSID& co90_Monitor::getIID() const 
{
  return m_iid;
}

inline  void co90_Monitor::setCurrent(int dispId, unsigned char* lpMethodName) {
  if ( dispId <size() ) {
    m_currInfo = &m_monInfoArray[dispId].m_monInfo;
    if (0 == m_currInfo->cmiCallCnt_gg00) {
      memcpy (&m_currInfo->cmiMethodName_gg00[0], lpMethodName,
        sizeof (m_currInfo->cmiMethodName_gg00));
    }
    ++m_currInfo->cmiCallCnt_gg00;
  }
  else {
    m_currInfo = NULL;
  }
}

inline co90_Monitor::Iterator::Iterator(co90_Monitor* monitor) : m_monitor(monitor), m_idx(-1) 
{
  operator++();
}

inline int co90_Monitor::Iterator::getDispId() const {
  return m_idx;
}

inline void co90_Monitor::Iterator::operator++() 
{
  ++m_idx;
  while ((m_idx < m_monitor->size()) && (0 == m_monitor->m_monInfoArray[m_idx].m_monInfo.cmiCallCnt_gg00)) {
    ++m_idx;
  }
}

inline co90_Monitor::Iterator::operator bool() const
{
  return (m_idx < m_monitor->size());
}

inline co90_MonitorInfo& co90_Monitor::Iterator::operator()() const
{
  return m_monitor->m_monInfoArray[m_idx];
}

#endif  /* GCO90_H */

==> sapdb-source-7.4.03.32.tgz: V74_03_32/SAPDB_ORG/sys/src/co/gco90.h <==
/* @lastChanged: "1998-05-27  11:02"
 * @filename:    gco90.h
 * @purpose:     "DBProcedure: Monitor"
 * @release:     7.1.0.0
 * @see:         ""
 *
 * @copyright:   "Copyright by SAP AG, 1998"


    ========== licence begin LGPL
    Copyright (C) 2000 SAP AG

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
    ========== licence end

 */


#ifndef GCO90_H
#define GCO90_H

/*===========================================================================*
 *  INCLUDES                                                                 *
 *===========================================================================*/

#include "ggg00.h"

/*===========================================================================*
 *  DEFINES                                                                  *
 *===========================================================================*/

#define LIVECACHEDBP_VERSION    "LiveCacheDbproc.h 7.1.0.0   DATE: 1998-04-27"

/*===========================================================================*
 *  MACROS                                                                   *
 *===========================================================================*/


/*===========================================================================*
 *  CLASSES, STRUCTURES, TYPES, UNIONS ...                                   *
 *===========================================================================*/

struct co90_MonitorInfo {
  tgg00_COMMonitorInfo m_monInfo;
  void Reset() {
    m_monInfo.cmiCallCnt_gg00 = 0;
    for (int ix = cmiRunTime; ix <= cmiCacheSize; ix++) {
      m_monInfo.cmiCounters_gg00[ix] = (tsp00_8ByteCounter) 0.0;
    }
    m_monInfo.cmiCounters_gg00[cmiRunTimeMin           ] = csp_maxint4;
    m_monInfo.cmiCounters_gg00[cmiMinWaitNewConsView   ] = csp_maxint4;
	  m_monInfo.cmiCounters_gg00[cmiMallocMin            ] = csp_maxint4;
    m_monInfo.cmiCounters_gg00[cmiFreeMin              ] = csp_maxint4;
    m_monInfo.cmiCounters_gg00[cmiDeltaMin             ] = csp_maxint4;
	  m_monInfo.cmiCounters_gg00[cmiMinReadApplCallbacks ] = csp_maxint4;
    m_monInfo.cmiCounters_gg00[cmiMinStreamRowsRead    ] = csp_maxint4;
    m_monInfo.cmiCounters_gg00[cmiMinWriteApplCallbacks] = csp_maxint4;
    m_monInfo.cmiCounters_gg00[cmiMinStreamRowsWrite   ] = csp_maxint4;
  }
  co90_MonitorInfo () {
    Reset();    
  }
};

class co90_Monitor {
public :
  class Iterator {
  public :
    Iterator(co90_Monitor*);
    inline int getDispId() const;
    inline void operator++();
    inline operator bool() const;
    inline co90_MonitorInfo& operator()() const;
  private :
    co90_Monitor* m_monitor;
    int           m_idx;
  };
 
  co90_Monitor(CLSID& IID, long numEntries) : m_size(numEntries), m_iid(IID),  m_currInfo(NULL) {
    /*PTS 1103516 FF*/
    m_monInfoArray = new co90_MonitorInfo[numEntries];
    if (NULL == m_monInfoArray ) 
      m_size = 0;
  }

  /*PTS 1103516 FF*/
  ~co90_Monitor() {
    delete [] m_monInfoArray;
  }

  void Reset () {
    for (int ix = 0; ix < m_size; ix++) {
      (m_monInfoArray+ix)->Reset();
    }
  }

  /* PTS 1112275 FF                                                    */
  /* function to handle InfoArray with a dimension, which is too small */
  /* this may occurr when dlopen doesn't support RTLD_LOCAL            */
  void increaseInfoArray( long num )
  {
	  if ( num > size() ) {
		  co90_MonitorInfo*  newInfoArray = new co90_MonitorInfo[num];
		  if ( newInfoArray ) {
        int i;
			  for ( i = 0; i < m_size; i++ )
				  newInfoArray[ i ] = m_monInfoArray[ i ];
			  for ( i = m_size; i < num; i++ )
				  newInfoArray[i].Reset();
			  m_size = num;
			  delete [] m_monInfoArray;
			  m_monInfoArray = newInfoArray;
		  }
	  }
  }

  inline Iterator begin();

  inline void callEpilog(int dispId, int runTime);

  tsp00_Int4 size() {
    return m_size;
  }

  inline tgg00_COMMonitorInfo* getCurrent();

  inline const CLSID& getIID() const;

  inline void setCurrent(int dispId, unsigned char* lpMethodName);

  co90_MonitorInfo& operator[](int idx) {
    return m_monInfoArray[idx];
  }

private :
  co90_MonitorInfo*     m_monInfoArray;   /*PTS 1103516 FF*/
  tsp00_Int4            m_size;
  CLSID                 m_iid;
  tgg00_COMMonitorInfo* m_currInfo;
 
  friend class co90_Monitor::Iterator;
};

/*===========================================================================*
 *  FUNCTION PROTOTYPES                                                      *
 *===========================================================================*/

inline co90_Monitor::Iterator co90_Monitor::begin() {
  Iterator iter(this);
  return iter;
}

inline void co90_Monitor::callEpilog(int dispId, int runTime)
{
  if ( dispId < size() ) {
    if (runTime > m_currInfo->cmiCounters_gg00[cmiRunTimeMax]) {
      m_currInfo->cmiCounters_gg00[cmiRunTimeMax] = runTime;
    }
    if (runTime < m_currInfo->cmiCounters_gg00[cmiRunTimeMin]) {
      m_currInfo->cmiCounters_gg00[cmiRunTimeMin] = runTime;
    }
    m_currInfo->cmiCounters_gg00[cmiRunTime] += runTime;
    m_currInfo = NULL;
  }
}

inline tgg00_COMMonitorInfo* co90_Monitor::getCurrent() {
  return m_currInfo;
}

inline const CLSID& co90_Monitor::getIID() const 
{
  return m_iid;
}

inline  void co90_Monitor::setCurrent(int dispId, unsigned char* lpMethodName) {
  if ( dispId <size() ) {
    m_currInfo = &m_monInfoArray[dispId].m_monInfo;
    if (0 == m_currInfo->cmiCallCnt_gg00) {
      memcpy (&m_currInfo->cmiMethodName_gg00[0], lpMethodName,
        sizeof (m_currInfo->cmiMethodName_gg00));
    }
    ++m_currInfo->cmiCallCnt_gg00;
  }
  else {
    m_currInfo = NULL;
  }
}

inline co90_Monitor::Iterator::Iterator(co90_Monitor* monitor) : m_monitor(monitor), m_idx(-1) 
{
  operator++();
}

inline int co90_Monitor::Iterator::getDispId() const {
  return m_idx;
}

inline void co90_Monitor::Iterator::operator++() 
{
  ++m_idx;
  while ((m_idx < m_monitor->size()) && (0 == m_monitor->m_monInfoArray[m_idx].m_monInfo.cmiCallCnt_gg00)) {
    ++m_idx;
  }
}

inline co90_Monitor::Iterator::operator bool() const
{
  return (m_idx < m_monitor->size());
}

inline co90_MonitorInfo& co90_Monitor::Iterator::operator()() const
{
  return m_monitor->m_monInfoArray[m_idx];
}

#endif  /* GCO90_H */
